var Protocol = function()
{
	var protocol = {};
	var debug = 0;
	
	var QUANTITY_REGULAR = 1.1;
	var LOW_QUANTITY_WARNING_FACTOR = 2;

	// Page initialization
	protocol.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'protocol')
		{
			Buttons.addClickHandler('print-button', Wizard.goPrint);	

			var libPlateId = Data.int('page-data', 'libplate');

			var url = '../LibPrep.servlet?ID='+App.getSessionId();
			url += '&cmd=GetLibraryInfoForPlate&bioplate='+libPlateId;		
			Wizard.showLoadingAnimation('Loading Library bioplate information...');
			Wizard.asyncJsonRequest(url, protocol.initializeProtocol);
		}
		else
		{
			Events.addEventHandler('bioplate', 'change', protocol.bioplateOnChange);
			Events.addEventHandler('listview', 'click', protocol.viewProtocol);
			Events.addEventHandler('plateview', 'click', protocol.viewProtocol);

			var request = Ajax.getXmlHttpRequest();
			var url = '../LibPrep.servlet?ID='+App.getSessionId();
			url += '&cmd=GetConfigurationsForCaliperRunParametersExporter';
			Wizard.asyncJsonRequest(url, protocol.initExportLinks);
			
			var url = '../LibPrep.servlet?ID='+App.getSessionId();
			url += '&cmd=GetLibraryPlatesForLibPrep';
			Wizard.showLoadingAnimation('Loading Library plates...');
			Wizard.asyncJsonRequest(url, protocol.initializeStep1);
		}
	}


	protocol.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];	

		var bioplates = response.bioplates;
		var plates = frm.bioplate;
		if (bioplates.length > 0)
		{
			for (var i=0; i < bioplates.length; i++)
			{
				var bioplate = bioplates[i];
				var option = new Option(bioplate.name, bioplate.id);
				option.bioplate = bioplate;
				plates.options[plates.length] = option;
			}
			bioplateIsValid = true;
			protocol.bioplateOnChange();
		}
		else
		{
			Wizard.setFatalError('No Library bioplates available for processing.');
			return;
		}
		
		Doc.show('step-1');
	}
	
	protocol.initExportLinks = function(response)
	{
		var configurations = response.configurations;
		var caliper96 = protocol.makeExportLink('Sample names for Caliper (96)', 'CALIPER', 'SAMPLE_NAMES');
		var caliper384 = protocol.makeExportLink('Sample names for Caliper (384)', 'CALIPER', 'SAMPLE_NAMES_384');
		for (var i = 0; i < configurations.length; i++)
		{
			var config = configurations[i];
			var btnHtml = protocol.makeExportLink('Run parameters: '+Strings.encodeTags(config.name), 'CALIPER', config.id);
			if (config.name.indexOf('384') > 0)
			{
				caliper384 += btnHtml;
			}
			else
			{
				caliper96 += btnHtml;
			}
		}
		Doc.element('exportCaliper96').innerHTML = caliper96;
		Doc.element('exportCaliper384').innerHTML = caliper384;

		var qubit = protocol.makeExportLink('Sample names for Qubit', 'QUBIT', 'SAMPLE_NAMES');
		Doc.element('exportQubit').innerHTML = qubit;
		
		var links = Doc.element('export').getElementsByClassName('link');
		for (var linkNo = 0; linkNo < links.length; linkNo++)
		{
			Events.addEventHandler(links[linkNo], 'click', protocol.exportFile);
		}
	}
	
	protocol.makeExportLink = function(title, hardware, exporter)
	{
		var btn = '';
		btn += '<span class="link" data-hardware="'+hardware+'" data-exporter="'+exporter+'">';
		btn += '<img src="../images/export.png">&nbsp;';
		btn += title+'</span><br>\n';
		return btn;
	}
	
	// Add pools to the pools list based on the bioplate selection
	protocol.bioplateOnChange = function()
	{
		var frm = document.forms['reggie'];
		var bioplate = frm.bioplate[frm.bioplate.selectedIndex].bioplate;
		Doc.element('comments').innerHTML = Strings.encodeTags(bioplate.comments);
		frm.poolSchema.value = bioplate.poolSchema;
		frm.barcodeVariant.value = bioplate.barcodeVariant;
	}
	

	protocol.viewProtocol = function(event)
	{
		var frm = document.forms['reggie'];
		if (frm.bioplate && !frm.bioplate.disabled)
		{
			frm.view.value = Data.get(event.currentTarget, 'protocol-type');
			frm.submit();
		}
	}

	protocol.exportFile = function(event)
	{
		var frm = document.forms['reggie'];
		if (frm.bioplate && !frm.bioplate.disabled)
		{
			var bioPlateId = frm.bioplate[frm.bioplate.selectedIndex].value;
		
			var hardware = Data.get(event.currentTarget, 'hardware');
			var exporter = Data.get(event.currentTarget, 'exporter');
			
			var url = '../LibPrep.servlet?ID='+App.getSessionId();
			if (hardware == 'CALIPER')
			{
				if (exporter == 'SAMPLE_NAMES')
				{
					url += '&cmd=ExportSampleNamesForCaliper&bioPlateId='+bioPlateId;
				}
				else if (exporter == 'SAMPLE_NAMES_384')
				{
					url += '&cmd=ExportSampleNamesForCaliper&remapTo384=1&bioPlateId='+bioPlateId;
				}
				else
				{
					url += '&cmd=ExportCaliperRunParameters&bioPlateId='+bioPlateId+'&configurationId='+exporter;
				}
			}
			else if (hardware == 'QUBIT')
			{
				if (exporter == 'SAMPLE_NAMES')
				{
					url += '&cmd=ExportSampleNamesForQubit&bioPlateId='+bioPlateId;
				}
				else
				{
					alert('not implemented');
					return;
				}
			}
			window.location = url;
		}
	}

	protocol.initializeProtocol = function(response)
	{
		var list = response.libraries;


		var schema = PoolSchema.getById(Data.get('page-data', 'schema'));
		var barcodeVariant = PoolSchema.getBarcodeVariantByName(schema, Data.get('page-data', 'barcode-variant'));

		// Pre-process the Library items
		for (var i = 0; i < list.length; i++)
		{
			protocol.checkAndPreProcessLibrary(list[i], schema, barcodeVariant);
		}
		
		var view = Data.get('page-data', 'view');
		if (view == 'list')
		{
			protocol.viewAsList(list, schema, barcodeVariant);
		}
		else
		{
			protocol.viewAsPlate(list, schema, barcodeVariant)
		}

	}
	
	/**
		Find the lowest used quantity. This is the QUANTITY_REGULAR
		value. RNA with larger used quantities are assumed to be selected
		for QC.
	*/
	protocol.findLowestUsedQuantity = function(mrnaList)
	{
		var lowestQuantity = mrnaList[0].rna.usedQuantity;
		for (var i = 1; i < mrnaList.length; i++)
		{
			var rna = mrnaList[i].rna;
			if (rna.usedQuantity < lowestQuantity)
			{
				lowestQuantity = rna.usedQuantity;
			}
		}
		return lowestQuantity;
	}

	
	protocol.checkAndPreProcessLibrary = function(lib, schema, barcodeVariant)
	{
		var well = lib.bioWell;
		var remarks = [];

		// Check if default barcode has been modified
		var indexSet = barcodeVariant.indexSets[well.column];
		if (indexSet)
		{
			var defaultBarcode = indexSet.barcodes[well.row];
			if (defaultBarcode && lib.barcode.name != defaultBarcode)
			{
				remarks[remarks.length] = 'Modified barcode';
				lib.barcode.modified = true;
			}
		}
		lib.remarks = remarks;
	}

	protocol.viewAsPlate = function(list, schema, barcodeVariant)
	{
		var rows = Data.get('page-data', 'rows');
		var columns = Data.get('page-data', 'columns');
		Plate.init(rows, columns, schema, WellPainter);
		
		for (var i = 0; i < list.length; i++)
		{
			var lib = list[i];
			var well = lib.bioWell;
			Plate.getWell(well.row, well.column).setExtract(lib);
		}

		WellPainter.barcodeVariant = barcodeVariant;
		Plate.paint(Plate.getWells());
		PoolSchema.buildPoolTableRow(schema, columns);

		Doc.show('all-protocol');
	}

	protocol.viewAsList = function(list, schema, barcodeVariant)
	{
		var lastPoolNum = -1;
		for (var i = 0; i < list.length; i++)
		{
			var lib = list[i];
			var well = lib.bioWell;
	
			var idSuffix = well.column + '.' + well.row;
			Doc.removeClass('row.'+idSuffix, 'empty');
			Doc.element('lib.'+idSuffix).innerHTML = Strings.encodeTags(lib.name);
			Doc.element('barcode.'+idSuffix).innerHTML = Strings.encodeTags(lib.barcode.name);
			
			var indexSet = barcodeVariant.indexSets[well.column];
			if (indexSet)
			{
				var color = lib.barcode.modified ? 'bg-modified' : indexSet.color;
				Doc.addClass('barcode.'+idSuffix, color);
			}
			
			Doc.element('remarks.'+idSuffix).innerHTML = lib.remarks.join('; ');
		}

		Doc.show('all-protocol');
	}
	
	
	return protocol;
}();

Doc.onLoad(Protocol.initPage);


var WellPainter = function()
{
	var painter = {};
	
	painter.getClassNameForWell = function(well, schema)
	{
		var cls = '';
		var indexSet = painter.barcodeVariant.indexSets[well.column];
		if (indexSet)
		{
			var lib = well.extract;
			cls += lib && lib.barcode.modified ? 'bg-modified' : indexSet.color;
		}
		return cls;
	}
	
	painter.getWellText = function(well, schema)
	{
		var text = '';
		var lib = well.extract;
		if (lib)
		{
			var name = lib.name;
			var i = name.indexOf('.m');
			text += '<div class="lib">'+Strings.encodeTags(name.substring(0, i))+'.<br>&nbsp;'+Strings.encodeTags(name.substring(i))+'</div>';
			text += '<div class="barcode">'+Strings.encodeTags(lib.barcode.name)+'</div>';
			text += '<div class="remarks">'+ lib.remarks.join('; ') + '</div>';
		}
		return text;
	}

	return painter;
}();
